@@ -19,13 +19,14 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 from log import m18n, m18nc, logDebug
-from common import LIGHTSOURCES, Internal, isAlive, ZValues, Debug
+from common import LIGHTSOURCES, Internal, isAlive, ZValues, Debug, WINDS
+from common import unicode, nativeString
 from twisted.internet.defer import succeed
 
-from qt import Qt, QMetaObject
+from qt import Qt, QMetaObject, variantValue
 from qt import QGraphicsScene, QGraphicsItem, QGraphicsRectItem, QPen, QColor
 
-from zope.interface import implements # pylint: disable=unused-import
+from zope.interface import implements  # pylint: disable=unused-import
 
 from dialogs import QuestionYesNo
 from guiutil import decorateWindow
@@ -35,11 +36,14 @@ from meld import Meld
 from humanclient import HumanClient
 from uitile import UITile
 from uiwall import UIWall
-from animation import Animated, afterCurrentAnimationDo
+from animation import MoveImmediate, afterQueuedAnimations
 from scoringdialog import ScoringDialog
 
+
 class FocusRect(QGraphicsRectItem):
+
     """show a focusRect with blue border around focused tile or meld"""
+
     def __init__(self):
         QGraphicsRectItem.__init__(self)
         pen = QPen(QColor(Qt.blue))
@@ -58,21 +62,24 @@ class FocusRect(QGraphicsRectItem):
     def board(self, value):
         """assign and show/hide as needed"""
         if value and not isAlive(value):
-            logDebug('assigning focusRect to a non-alive board %s/%s' % (type(value), value))
+            logDebug(
+                u'assigning focusRect to a non-alive board %s/%s' %
+                (type(value), value))
             return
         if value:
             self._board = value
             self.refresh()
 
-    def __refreshNow(self, dummy):
-        """show/hide on correct position"""
+    @afterQueuedAnimations
+    def refresh(self, deferredResult=None):
+        """show/hide on correct position after queued animations end"""
         board = self.board
         if not isAlive(board) or not isAlive(self):
             if isAlive(self):
                 self.setVisible(False)
             return
         rect = board.tileFaceRect()
-        rect.setWidth(rect.width()*board.focusRectWidth())
+        rect.setWidth(rect.width() * board.focusRectWidth())
         self.setRect(rect)
         self.setRotation(board.sceneRotation())
         self.setScale(board.scale())
@@ -81,28 +88,29 @@ class FocusRect(QGraphicsRectItem):
             self.setPos(board.focusTile.pos)
         game = Internal.scene.game
         self.setVisible(board.isVisible() and bool(board.focusTile)
-            and board.isEnabled() and board.hasFocus and bool(game) and not game.autoPlay)
+                        and board.isEnabled() and board.hasFocus and bool(game) and not game.autoPlay)
 
-    def refresh(self):
-        """show/hide on correct position after current animations end"""
-        afterCurrentAnimationDo(self.__refreshNow)
 
 class SceneWithFocusRect(QGraphicsScene):
+
     """our scene with a potential Qt bug fix. FocusRect is a blue frame around a tile or meld"""
+
     def __init__(self):
         QGraphicsScene.__init__(self)
         self.focusRect = FocusRect()
         self.addItem(self.focusRect)
 
     def focusInEvent(self, event):
-        """work around a qt bug. See https://bugreports.qt-project.org/browse/QTBUG-32890
+        """
+        Work around a qt bug. See U{https://bugreports.qt-project.org/browse/QTBUG-32890}.
         This can be reproduced as follows:
-           ./kajongg.py --game=2/E2 --demo --ruleset=BMJA
+         - ./kajongg.py --game=2/E2 --demo --ruleset=BMJA
                such that the human player is the first one to discard a tile.
-           wait until the main screen has been built
-           click with the mouse into the middle of that window
-           press left arrow key
-           this will violate the assertion in UITile.keyPressEvent """
+         - wait until the main screen has been built
+         - click with the mouse into the middle of that window
+         - press left arrow key
+         - this will violate the assertion in UITile.keyPressEvent.
+        """
         prev = self.focusItem()
         QGraphicsScene.focusInEvent(self, event)
         if prev and bool(prev.flags() & QGraphicsItem.ItemIsFocusable) and prev != self.focusItem():
@@ -118,7 +126,9 @@ class SceneWithFocusRect(QGraphicsScene):
         """get / set the board that has its focusRect shown"""
         self.focusRect.board = board
 
+
 class GameScene(SceneWithFocusRect):
+
     """the game field"""
     # pylint: disable=too-many-instance-attributes
 
@@ -127,11 +137,11 @@ class GameScene(SceneWithFocusRect):
         self.mainWindow = parent
         self._game = None
         super(GameScene, self).__init__()
-        self.showShadows = True
 
         self.scoreTable = None
         self.explainView = None
         self.setupUi()
+        Internal.Preferences.addWatch('showShadows', self.showShadowsChanged)
 
     @property
     def game(self):
@@ -157,19 +167,24 @@ class GameScene(SceneWithFocusRect):
         self.mainWindow.updateGUI()
         self.mainWindow.adjustView()
 
+    def showShadowsChanged(self, oldValue, newValue):
+        for uiTile in self.graphicsTileItems():
+            uiTile.setClippingFlags()
+        self.applySettings()
+
     def handSelectorChanged(self, handBoard):
         """update all relevant dialogs"""
         if self.game and not self.game.finished():
-            self.game.wall.decoratePlayer(handBoard.player) # pylint:disable=no-member
-        # first decorate walls - that will compute player.handBoard for explainView
+            self.game.wall.decoratePlayer(
+                handBoard.player)  # pylint:disable=no-member
+        # first decorate walls - that will compute player.handBoard for
+        # explainView
         if self.explainView:
             self.explainView.refresh()
 
     def setupUi(self):
         """prepare scene"""
         # pylint: disable=too-many-statements
-        self.tileset = None # just for pylint
-        self.tilesetName = Internal.Preferences.tilesetName
         self.windTileset = Tileset(Internal.Preferences.windTilesetName)
 
     def showWall(self):
@@ -184,47 +199,19 @@ class GameScene(SceneWithFocusRect):
         """adjust the view such that exactly the wanted things are displayed
         without having to scroll"""
         if self.game:
-            with Animated(False):
+            with MoveImmediate():
                 self.game.wall.decorate()
                 for uiTile in self.game.wall.tiles:
                     if uiTile.board:
                         uiTile.board.placeTile(uiTile)
 
-    @property
-    def tilesetName(self):
-        """the name of the current tileset"""
-        return self.tileset.desktopFileName
-
-    @tilesetName.setter
-    def tilesetName(self, name):
-        """the name of the current tileset"""
-        self.tileset = Tileset(name)
-
     def applySettings(self):
         """apply preferences"""
-        self.mainWindow.actionAngle.setEnabled(bool(self.game) and Internal.Preferences.showShadows)
-        with Animated(False):
-            if self.tilesetName != Internal.Preferences.tilesetName:
-                self.tilesetName = Internal.Preferences.tilesetName
-                if self.game:
-                    self.game.wall.tileset = self.tileset
-                for item in self.nonTiles():
-                    try:
-                        item.tileset = self.tileset
-                    except AttributeError:
-                        continue
-                # change players last because we need the wall already to be repositioned
-            if self.game:
-                for player in self.game.players:
-                    if player.handBoard:
-                        player.handBoard.rearrangeMelds = Internal.Preferences.rearrangeMelds
-            if self.showShadows is None or self.showShadows != Internal.Preferences.showShadows:
-                self.showShadows = Internal.Preferences.showShadows
-                if self.game:
-                    wall = self.game.wall
-                    wall.showShadows = self.showShadows
-                for uiTile in self.graphicsTileItems():
-                    uiTile.setClippingFlags()
+        self.mainWindow.actionAngle.setEnabled(
+            bool(self.game) and Internal.Preferences.showShadows)
+        with MoveImmediate():
+            for item in self.nonTiles():
+                item.tileset = Tileset.activeTileset()
 
     def prepareHand(self):
         """redecorate wall"""
@@ -239,7 +226,8 @@ class GameScene(SceneWithFocusRect):
         for action in [mainWindow.actionScoreGame, mainWindow.actionPlayGame]:
             action.setEnabled(not bool(game))
         mainWindow.actionAbortGame.setEnabled(bool(game))
-        mainWindow.actionAngle.setEnabled(bool(game) and self.showShadows)
+        mainWindow.actionAngle.setEnabled(
+            bool(game) and Internal.Preferences.showShadows)
         for view in [self.explainView, self.scoreTable]:
             if view:
                 view.refresh()
@@ -250,7 +238,7 @@ class GameScene(SceneWithFocusRect):
         oldIdx = LIGHTSOURCES.index(self.game.wall.lightSource)
         return LIGHTSOURCES[(oldIdx + 1) % 4]
 
-    def changeAngle(self, dummyResult):
+    def changeAngle(self):
         """change the lightSource"""
         self.game.wall.lightSource = self.newLightSource()
         self.focusRect.refresh()
@@ -286,18 +274,20 @@ class GameScene(SceneWithFocusRect):
             self.removeItem(item)
         self.focusRect.hide()
 
+
 class PlayingScene(GameScene):
+
     """scene with a playing game"""
+
     def __init__(self, parent):
-        super(PlayingScene, self).__init__(parent)
-        self.game = None
+        self._game = None
         self.__startingGame = True
         self._clientDialog = None
 
-        self.setupUi()
+        super(PlayingScene, self).__init__(parent)
 
     @GameScene.game.setter
-    def game(self, value): # pylint: disable=arguments-differ
+    def game(self, value):  # pylint: disable=arguments-differ
         game = self._game
         changing = value != game
         GameScene.game.fset(self, value)
@@ -344,7 +334,6 @@ class PlayingScene(GameScene):
         self.setObjectName("PlayingField")
 
         self.discardBoard = DiscardBoard()
-        #self.discardBoard.setVisible(True)
         self.addItem(self.discardBoard)
 
         self.adjustView()
@@ -386,7 +375,7 @@ class PlayingScene(GameScene):
         """adjust the view such that exactly the wanted things are displayed
         without having to scroll"""
         if self.game:
-            with Animated(False):
+            with MoveImmediate():
                 self.discardBoard.maximize()
         GameScene.adjustView(self)
 
@@ -405,16 +394,19 @@ class PlayingScene(GameScene):
     def applySettings(self):
         """apply preferences"""
         GameScene.applySettings(self)
-        self.discardBoard.showShadows = self.showShadows
+        self.discardBoard.showShadows = Internal.Preferences.showShadows
 
     def toggleDemoMode(self, checked):
         """switch on / off for autoPlay"""
         if self.game:
-            self.focusRect.refresh() # show/hide it
+            self.focusRect.refresh()  # show/hide it
             self.game.autoPlay = checked
             if checked and self.clientDialog:
-                self.clientDialog.proposeAction() # an illegal action might have focus
-                self.clientDialog.selectButton() # select default, abort timeout
+                self.clientDialog.proposeAction()
+                                                # an illegal action might have
+                                                # focus
+                self.clientDialog.selectButton()
+                                               # select default, abort timeout
 
     def updateSceneGUI(self):
         """update some actions, all auxiliary windows and the statusbar"""
@@ -424,8 +416,10 @@ class PlayingScene(GameScene):
         game = self.game
         mainWindow = self.mainWindow
         if not game:
-            connections = list(x.connection for x in HumanClient.humanClients if x.connection)
-            title = ', '.join('{name}/{url}'.format(name=x.username, url=x.url) for x in connections)
+            connections = list(
+                x.connection for x in HumanClient.humanClients if x.connection)
+            title = u', '.join(u'{name}/{url}'.format(name=x.username, url=x.url)
+                               for x in connections)
             if title:
                 decorateWindow(mainWindow, title)
         else:
@@ -436,28 +430,33 @@ class PlayingScene(GameScene):
         self.discardBoard.setVisible(bool(game))
         mainWindow.actionAutoPlay.setEnabled(not self.startingGame)
         mainWindow.actionChat.setEnabled(bool(game) and bool(game.client)
-            and bool(game.client.connection)
-            and not game.client.connection.url.isLocalGame and not self.startingGame)
-            # chatting on tables before game started works with chat button per table
-        mainWindow.actionChat.setChecked(mainWindow.actionChat.isEnabled() and bool(game.client.table.chatWindow))
+                                         and bool(game.client.connection)
+                                         and not game.client.connection.url.isLocalGame and not self.startingGame)
+            # chatting on tables before game started works with chat button per
+            # table
+        mainWindow.actionChat.setChecked(
+            mainWindow.actionChat.isEnabled(
+            ) and bool(
+                game.client.table.chatWindow))
 
-    def changeAngle(self, result):
+    def changeAngle(self):
         """now that no animation is running, really change"""
         self.discardBoard.lightSource = self.newLightSource()
-        GameScene.changeAngle(self, result)
+        GameScene.changeAngle(self)
+
 
 class ScoringScene(GameScene):
+
     """a scoring game"""
     # pylint: disable=too-many-instance-attributes
 
     def __init__(self, parent=None):
-        super(ScoringScene, self).__init__(parent)
         self.scoringDialog = None
-        self.setupUi()
+        super(ScoringScene, self).__init__(parent)
         self.selectorBoard.hasFocus = True
 
     @GameScene.game.setter
-    def game(self, value): # pylint: disable=arguments-differ
+    def game(self, value):  # pylint: disable=arguments-differ
         game = self._game
         changing = value != game
         GameScene.game.fset(self, value)
@@ -490,7 +489,7 @@ class ScoringScene(GameScene):
                 self.game = None
             return result
         if Debug.quit:
-            logDebug('ScoringScene.abort invoked')
+            logDebug(u'ScoringScene.abort invoked')
         if not self.game:
             return succeed(True)
         elif self.game.finished():
@@ -520,24 +519,26 @@ class ScoringScene(GameScene):
         """keyboard navigation in a scoring game"""
         mod = event.modifiers()
         key = event.key()
-        wind = chr(key%128)
-        moveCommands = m18nc('kajongg:keyboard commands for moving tiles to the players ' \
-            'with wind ESWN or to the central tile selector (X)', 'ESWNX')
+        wind = chr(key % 128)
+        windsX = WINDS + u'X'
+        moveCommands = m18nc('kajongg:keyboard commands for moving tiles to the players '
+                             'with wind ESWN or to the central tile selector (X)', windsX)
         uiTile = self.focusItem()
         if wind in moveCommands:
             # translate i18n wind key to ESWN:
-            wind = 'ESWNX'[moveCommands.index(wind)]
-            self.__moveTile(uiTile, wind, bool(mod &Qt.ShiftModifier))
+            wind = windsX[moveCommands.index(wind)]
+            self.__moveTile(uiTile, wind, bool(mod & Qt.ShiftModifier))
             return True
         if key == Qt.Key_Tab and self.game:
             tabItems = [self.selectorBoard]
-            tabItems.extend(list(p.handBoard for p in self.game.players if p.handBoard.uiTiles))
+            tabItems.extend(
+                list(p.handBoard for p in self.game.players if p.handBoard.uiTiles))
             tabItems.append(tabItems[0])
             currentBoard = uiTile.board
             currIdx = 0
-            while tabItems[currIdx] != currentBoard and currIdx < len(tabItems) -2:
+            while tabItems[currIdx] != currentBoard and currIdx < len(tabItems) - 2:
                 currIdx += 1
-            tabItems[currIdx+1].hasFocus = True
+            tabItems[currIdx + 1].hasFocus = True
             return True
 
     def keyPressEvent(self, event):
@@ -553,15 +554,10 @@ class ScoringScene(GameScene):
         """adjust the view such that exactly the wanted things are displayed
         without having to scroll"""
         if self.game:
-            with Animated(False):
+            with MoveImmediate():
                 self.selectorBoard.maximize()
         GameScene.adjustView(self)
 
-    def applySettings(self):
-        """apply preferences"""
-        GameScene.applySettings(self)
-        self.selectorBoard.showShadows = self.showShadows
-
     def prepareHand(self):
         """redecorate wall"""
         GameScene.prepareHand(self)
@@ -581,10 +577,10 @@ class ScoringScene(GameScene):
         self.selectorBoard.setVisible(bool(game))
         self.selectorBoard.setEnabled(bool(game))
 
-    def changeAngle(self, result):
+    def changeAngle(self):
         """now that no animation is running, really change"""
         self.selectorBoard.lightSource = self.newLightSource()
-        GameScene.changeAngle(self, result)
+        GameScene.changeAngle(self)
 
     def computeLastTile(self):
         """compile hand info into a string as needed by the scoring engine"""
@@ -599,5 +595,6 @@ class ScoringScene(GameScene):
             cbLastMeld = self.scoringDialog.cbLastMeld
             idx = cbLastMeld.currentIndex()
             if idx >= 0:
-                return Meld(str(cbLastMeld.itemData(idx).toString()))
+                return Meld(nativeString(
+                    variantValue(cbLastMeld.itemData(idx))))
         return Meld()
